@@ -7,15 +7,16 @@ from pathlib import Path
 from socket import AddressFamily, SocketKind
 from typing import Awaitable, List, Optional, Tuple, Union, cast, overload
 
-from ..abc import ConnectedUDPSocket, Event, SocketListener, SocketStream, UDPSocket
-from ..abc.sockets import IPAddressType, IPSockAddrType
+from .. import to_thread
+from ..abc import (
+    ConnectedUDPSocket, IPAddressType, IPSockAddrType, SocketListener, SocketStream, UDPSocket,
+    UNIXSocketStream)
 from ..streams.stapled import MultiListener
 from ..streams.tls import TLSStream
 from ._eventloop import get_asynclib
 from ._resources import aclose_forcefully
-from ._synchronization import create_event
+from ._synchronization import Event
 from ._tasks import create_task_group, move_on_after
-from ._threads import run_sync_in_worker_thread
 
 if sys.version_info >= (3, 8):
     from typing import Literal
@@ -82,9 +83,11 @@ async def connect_tcp(
 
 
 async def connect_tcp(
-    remote_host, remote_port, *, local_host=None, tls=False, ssl_context=None,
-    tls_standard_compatible=True, tls_hostname=None, happy_eyeballs_delay=0.25
-):
+    remote_host: IPAddressType, remote_port: int, *, local_host: Optional[IPAddressType] = None,
+    tls: bool = False, ssl_context: Optional[ssl.SSLContext] = None,
+    tls_standard_compatible: bool = True, tls_hostname: Optional[str] = None,
+    happy_eyeballs_delay: float = 0.25
+) -> Union[SocketStream, TLSStream]:
     """
     Connect to a host using the TCP protocol.
 
@@ -118,7 +121,7 @@ async def connect_tcp(
     # Placed here due to https://github.com/python/mypy/issues/7057
     connected_stream: Optional[SocketStream] = None
 
-    async def try_connect(remote_host: str, event: Event):
+    async def try_connect(remote_host: str, event: Event) -> None:
         nonlocal connected_stream
         try:
             stream = await asynclib.connect_tcp(remote_host, remote_port, local_address)
@@ -128,11 +131,11 @@ async def connect_tcp(
         else:
             if connected_stream is None:
                 connected_stream = stream
-                await tg.cancel_scope.cancel()
+                tg.cancel_scope.cancel()
             else:
                 await stream.aclose()
         finally:
-            await event.set()
+            event.set()
 
     asynclib = get_asynclib()
     local_address: Optional[IPSockAddrType] = None
@@ -171,9 +174,9 @@ async def connect_tcp(
     oserrors: List[OSError] = []
     async with create_task_group() as tg:
         for i, (af, addr) in enumerate(target_addrs):
-            event = create_event()
-            await tg.spawn(try_connect, addr, event)
-            async with move_on_after(happy_eyeballs_delay):
+            event = Event()
+            tg.start_soon(try_connect, addr, event)
+            with move_on_after(happy_eyeballs_delay):
                 await event.wait()
 
     if connected_stream is None:
@@ -183,7 +186,7 @@ async def connect_tcp(
     if tls or tls_hostname or ssl_context:
         try:
             return await TLSStream.wrap(connected_stream, server_side=False,
-                                        hostname=tls_hostname or remote_host,
+                                        hostname=tls_hostname or str(remote_host),
                                         ssl_context=ssl_context,
                                         standard_compatible=tls_standard_compatible)
         except BaseException:
@@ -193,7 +196,7 @@ async def connect_tcp(
     return connected_stream
 
 
-async def connect_unix(path: Union[str, PathLike]) -> SocketStream:
+async def connect_unix(path: Union[str, 'PathLike[str]']) -> UNIXSocketStream:
     """
     Connect to the given UNIX socket.
 
@@ -211,7 +214,7 @@ async def create_tcp_listener(
     *, local_host: Optional[IPAddressType] = None, local_port: int = 0,
     family: AnyIPAddressFamily = socket.AddressFamily.AF_UNSPEC, backlog: int = 65536,
     reuse_port: bool = False
-) -> MultiListener[SocketStream[IPSockAddrType]]:
+) -> MultiListener[SocketStream]:
     """
     Create a TCP socket listener.
 
@@ -233,7 +236,7 @@ async def create_tcp_listener(
     gai_res = await getaddrinfo(local_host, local_port, family=family,  # type: ignore[arg-type]
                                 type=socket.SOCK_STREAM,
                                 flags=socket.AI_PASSIVE | socket.AI_ADDRCONFIG)
-    listeners: List[SocketListener[IPSockAddrType]] = []
+    listeners: List[SocketListener] = []
     try:
         # The set() is here to work around a glibc bug:
         # https://sourceware.org/bugzilla/show_bug.cgi?id=14969
@@ -256,7 +259,7 @@ async def create_tcp_listener(
 
             raw_socket.bind(sockaddr)
             raw_socket.listen(backlog)
-            listener = asynclib.SocketListener(raw_socket)
+            listener = asynclib.TCPSocketListener(raw_socket)
             listeners.append(listener)
     except BaseException:
         for listener in listeners:
@@ -268,8 +271,8 @@ async def create_tcp_listener(
 
 
 async def create_unix_listener(
-        path: Union[str, PathLike], *, mode: Optional[int] = None,
-        backlog: int = 65536) -> SocketListener[str]:
+        path: Union[str, 'PathLike[str]'], *, mode: Optional[int] = None,
+        backlog: int = 65536) -> SocketListener:
     """
     Create a UNIX socket listener.
 
@@ -281,18 +284,25 @@ async def create_unix_listener(
         65536)
     :return: a listener object
 
+    .. versionchanged:: 3.0
+        If a socket already exists on the file system in the given path, it will be removed first.
+
     """
-    path = str(Path(path))
+    path_str = str(path)
+    path = Path(path)
+    if path.is_socket():
+        path.unlink()
+
     backlog = min(backlog, 65536)
     raw_socket = socket.socket(socket.AF_UNIX)
     raw_socket.setblocking(False)
     try:
-        await run_sync_in_worker_thread(raw_socket.bind, path, cancellable=True)
+        await to_thread.run_sync(raw_socket.bind, path_str, cancellable=True)
         if mode is not None:
-            await run_sync_in_worker_thread(chmod, path, mode, cancellable=True)
+            await to_thread.run_sync(chmod, path_str, mode, cancellable=True)
 
         raw_socket.listen(backlog)
-        return get_asynclib().SocketListener(raw_socket)
+        return get_asynclib().UNIXSocketListener(raw_socket)
     except BaseException:
         raw_socket.close()
         raise
@@ -309,7 +319,7 @@ async def create_udp_socket(
     making this socket suitable for providing UDP based services.
 
     :param family: address family (``AF_INET`` or ``AF_INET6``) – automatically determined from
-        ``interface`` or ``target_host`` if omitted
+        ``local_host`` if omitted
     :param local_host: IP address or host name of the local interface to bind to
     :param local_port: local port to bind to
     :param reuse_port: ``True`` to allow multiple sockets to bind to the same address/port
@@ -320,13 +330,16 @@ async def create_udp_socket(
     if family is AddressFamily.AF_UNSPEC and not local_host:
         raise ValueError('Either "family" or "local_host" must be given')
 
-    local_address: Optional[IPSockAddrType] = None
     if local_host:
         gai_res = await getaddrinfo(str(local_host), local_port, family=family,
                                     type=socket.SOCK_DGRAM,
                                     flags=socket.AI_PASSIVE | socket.AI_ADDRCONFIG)
         family = cast(AnyIPAddressFamily, gai_res[0][0])
         local_address = gai_res[0][-1]
+    elif family is AddressFamily.AF_INET6:
+        local_address = ('::', 0)
+    else:
+        local_address = ('0.0.0.0', 0)
 
     return await get_asynclib().create_udp_socket(family, local_address, None, reuse_port)
 
@@ -430,6 +443,9 @@ def wait_socket_readable(sock: socket.socket) -> Awaitable[None]:
     This does **NOT** work on Windows when using the asyncio backend with a proactor event loop
     (default on py3.8+).
 
+    .. warning:: Only use this on raw sockets that have not been wrapped by any higher level
+        constructs like socket streams!
+
     :param sock: a socket object
     :raises ~anyio.ClosedResourceError: if the socket was closed while waiting for the
         socket to become readable
@@ -447,6 +463,9 @@ def wait_socket_writable(sock: socket.socket) -> Awaitable[None]:
     This does **NOT** work on Windows when using the asyncio backend with a proactor event loop
     (default on py3.8+).
 
+    .. warning:: Only use this on raw sockets that have not been wrapped by any higher level
+        constructs like socket streams!
+
     :param sock: a socket object
     :raises ~anyio.ClosedResourceError: if the socket was closed while waiting for the
         socket to become writable
@@ -461,7 +480,9 @@ def wait_socket_writable(sock: socket.socket) -> Awaitable[None]:
 # Private API
 #
 
-def convert_ipv6_sockaddr(sockaddr):
+def convert_ipv6_sockaddr(
+    sockaddr: Union[Tuple[str, int, int, int], Tuple[str, int]]
+) -> Tuple[str, int]:
     """
     Convert a 4-tuple IPv6 socket address to a 2-tuple (address, port) format.
 
@@ -475,10 +496,11 @@ def convert_ipv6_sockaddr(sockaddr):
     """
     # This is more complicated than it should be because of MyPy
     if isinstance(sockaddr, tuple) and len(sockaddr) == 4:
-        if sockaddr[3]:
-            # Add scopeid to the address
-            return sockaddr[0] + '%' + str(sockaddr[3]), sockaddr[1]
+        host, port, flowinfo, scope_id = cast(Tuple[str, int, int, int], sockaddr)
+        if scope_id:
+            # Add scope_id to the address
+            return f"{host}%{scope_id}", port
         else:
-            return sockaddr[:2]
+            return host, port
     else:
-        return sockaddr
+        return cast(Tuple[str, int], sockaddr)
